@@ -12,6 +12,7 @@
 #    under the License.
 
 import json
+import logging
 import sys
 
 from cliff import command
@@ -21,6 +22,9 @@ from cliff import show
 from gnocchiclient import utils
 
 
+LOG_DEP = logging.getLogger('deprecated')
+
+
 class CliMetricWithResourceID(command.Command):
     def get_parser(self, prog_name):
         parser = super(CliMetricWithResourceID, self).get_parser(prog_name)
@@ -34,14 +38,37 @@ class CliMetricList(lister.Lister):
 
     COLS = ('id', 'archive_policy/name', 'name', 'unit', 'resource_id')
 
+    def get_parser(self, prog_name):
+        parser = super(CliMetricList, self).get_parser(prog_name)
+        parser.add_argument("--limit", type=int, metavar="<LIMIT>",
+                            help="Number of metrics to return "
+                            "(Default is server default)")
+        parser.add_argument("--marker", metavar="<MARKER>",
+                            help="Last item of the previous listing. "
+                            "Return the next results after this value")
+        parser.add_argument("--sort", action="append", metavar="<SORT>",
+                            help="Sort of metric attribute "
+                            "(example: user_id:desc-nullslast")
+        return parser
+
     def take_action(self, parsed_args):
-        metrics = utils.get_client(self).metric.list()
+        metrics = utils.get_client(self).metric.list(
+            **utils.get_pagination_options(parsed_args))
         for metric in metrics:
             utils.format_archive_policy(metric["archive_policy"])
             utils.format_move_dict_to_root(metric, "archive_policy")
         return utils.list2cols(self.COLS, metrics)
 
 
+class DeprecatedCliMetricList(CliMetricList):
+    """Deprecated: List metrics"""
+
+    def take_action(self, parsed_args):
+        LOG_DEP.warning('This command has been deprecated. '
+                        'Please use "metric list" instead.')
+        return super(DeprecatedCliMetricList, self).take_action(parsed_args)
+
+
 class CliMetricShow(CliMetricWithResourceID, show.ShowOne):
     """Show a metric"""
 
@@ -55,12 +82,23 @@ class CliMetricShow(CliMetricWithResourceID, show.ShowOne):
         metric = utils.get_client(self).metric.get(
             metric=parsed_args.metric,
             resource_id=parsed_args.resource_id)
-        utils.format_archive_policy(metric["archive_policy"])
-        utils.format_move_dict_to_root(metric, "archive_policy")
+        metric['archive_policy/name'] = metric["archive_policy"]["name"]
+        del metric['archive_policy']
+        del metric['created_by_user_id']
+        del metric['created_by_project_id']
         utils.format_resource_for_metric(metric)
         return self.dict2columns(metric)
 
 
+class DeprecatedCliMetricShow(CliMetricShow):
+    """Deprecated: Show a metric"""
+
+    def take_action(self, parsed_args):
+        LOG_DEP.warning('This command has been deprecated. '
+                        'Please use "metric show" instead.')
+        return super(DeprecatedCliMetricShow, self).take_action(parsed_args)
+
+
 class CliMetricCreateBase(show.ShowOne, CliMetricWithResourceID):
     def get_parser(self, prog_name):
         parser = super(CliMetricCreateBase, self).get_parser(prog_name)
@@ -69,12 +107,6 @@ class CliMetricCreateBase(show.ShowOne, CliMetricWithResourceID):
                             help="name of the archive policy")
         return parser
 
-    def take_action(self, parsed_args):
-        metric = utils.dict_from_parsed_args(parsed_args,
-                                             ["archive_policy_name",
-                                              "resource_id"])
-        return self._take_action(metric, parsed_args)
-
 
 class CliMetricCreate(CliMetricCreateBase):
     """Create a metric"""
@@ -88,18 +120,31 @@ class CliMetricCreate(CliMetricCreateBase):
                             help="unit of the metric")
         return parser
 
-    def _take_action(self, metric, parsed_args):
-        if parsed_args.name:
-            metric['name'] = parsed_args.name
-        if parsed_args.unit:
-            metric['unit'] = parsed_args.unit
-        metric = utils.get_client(self).metric.create(metric)
-        utils.format_archive_policy(metric["archive_policy"])
-        utils.format_move_dict_to_root(metric, "archive_policy")
+    def take_action(self, parsed_args):
+        metric = utils.get_client(self).metric._create_new(
+            archive_policy_name=parsed_args.archive_policy_name,
+            name=parsed_args.name,
+            resource_id=parsed_args.resource_id,
+            unit=parsed_args.unit,
+        )
         utils.format_resource_for_metric(metric)
+        if 'archive_policy' in metric:
+            metric['archive_policy/name'] = metric["archive_policy"]["name"]
+            del metric['archive_policy']
+        del metric['created_by_user_id']
+        del metric['created_by_project_id']
         return self.dict2columns(metric)
 
 
+class DeprecatedCliMetricCreate(CliMetricCreate):
+    """Deprecated: Create a metric"""
+
+    def take_action(self, parsed_args):
+        LOG_DEP.warning('This command has been deprecated. '
+                        'Please use "metric create" instead.')
+        return super(DeprecatedCliMetricCreate, self).take_action(parsed_args)
+
+
 class CliMetricDelete(CliMetricWithResourceID):
     """Delete a metric"""
 
@@ -115,7 +160,34 @@ class CliMetricDelete(CliMetricWithResourceID):
                 metric=metric, resource_id=parsed_args.resource_id)
 
 
-class CliMeasuresShow(CliMetricWithResourceID, lister.Lister):
+class DeprecatedCliMetricDelete(CliMetricDelete):
+    """Deprecated: Delete a metric"""
+
+    def take_action(self, parsed_args):
+        LOG_DEP.warning('This command has been deprecated. '
+                        'Please use "metric delete" instead.')
+        return super(DeprecatedCliMetricDelete, self).take_action(parsed_args)
+
+
+class CliMeasuresReturn(lister.Lister):
+    def get_parser(self, prog_name):
+        parser = super(CliMeasuresReturn, self).get_parser(prog_name)
+        parser.add_argument("--utc", help="Return timestamps as UTC",
+                            default=False,
+                            action="store_true")
+        return parser
+
+    @staticmethod
+    def format_measures_with_tz(parsed_args, measures):
+        if parsed_args.utc:
+            t = lambda x: x
+        else:
+            t = utils.dt_to_localtz
+        return [(t(dt).isoformat(), g, v) for dt, g, v in measures]
+
+
+class CliMeasuresShow(CliMetricWithResourceID, CliMeasuresReturn,
+                      lister.Lister):
     """Get measurements of a metric"""
 
     COLS = ('timestamp', 'granularity', 'value')
@@ -127,11 +199,18 @@ class CliMeasuresShow(CliMetricWithResourceID, lister.Lister):
         parser.add_argument("--aggregation",
                             help="aggregation to retrieve")
         parser.add_argument("--start",
+                            type=utils.parse_date,
                             help="beginning of the period")
         parser.add_argument("--stop",
+                            type=utils.parse_date,
                             help="end of the period")
         parser.add_argument("--granularity",
-                            help="granularity to retrieve (in seconds)")
+                            help="granularity to retrieve")
+        parser.add_argument("--refresh", action="store_true",
+                            help="force aggregation of all known measures")
+        parser.add_argument("--resample",
+                            help=("granularity to resample time-series to "
+                                  "(in seconds)"))
         return parser
 
     def take_action(self, parsed_args):
@@ -142,8 +221,10 @@ class CliMeasuresShow(CliMetricWithResourceID, lister.Lister):
             start=parsed_args.start,
             stop=parsed_args.stop,
             granularity=parsed_args.granularity,
+            refresh=parsed_args.refresh,
+            resample=parsed_args.resample
         )
-        return self.COLS, measures
+        return self.COLS, self.format_measures_with_tz(parsed_args, measures)
 
 
 class CliMeasuresAddBase(CliMetricWithResourceID):
@@ -158,7 +239,8 @@ class CliMeasuresAdd(CliMeasuresAddBase):
 
     def measure(self, measure):
         timestamp, __, value = measure.rpartition("@")
-        return {'timestamp': timestamp, 'value': float(value)}
+        return {'timestamp': utils.parse_date(timestamp).isoformat(),
+                'value': float(value)}
 
     def get_parser(self, prog_name):
         parser = super(CliMeasuresAdd, self).get_parser(prog_name)
@@ -199,13 +281,20 @@ class CliMetricsMeasuresBatch(CliMeasuresBatch):
 
 
 class CliResourcesMetricsMeasuresBatch(CliMeasuresBatch):
+    def get_parser(self, prog_name):
+        parser = super(CliResourcesMetricsMeasuresBatch, self).get_parser(
+            prog_name)
+        parser.add_argument("--create-metrics", action='store_true',
+                            help="Create unknown metrics"),
+        return parser
+
     def take_action(self, parsed_args):
         with parsed_args.file as f:
             utils.get_client(self).metric.batch_resources_metrics_measures(
-                json.load(f))
+                json.load(f), create_metrics=parsed_args.create_metrics)
 
 
-class CliMeasuresAggregation(lister.Lister):
+class CliMeasuresAggregation(CliMeasuresReturn):
     """Get measurements of aggregated metrics"""
 
     COLS = ('timestamp', 'granularity', 'value')
@@ -214,14 +303,18 @@ class CliMeasuresAggregation(lister.Lister):
         parser = super(CliMeasuresAggregation, self).get_parser(prog_name)
         parser.add_argument("-m", "--metric", nargs='+', required=True,
                             help="metrics IDs or metric name")
-        parser.add_argument("--aggregation",
-                            help="aggregation to retrieve")
+        parser.add_argument("--aggregation", help="granularity aggregation "
+                                                  "function to retrieve")
+        parser.add_argument("--reaggregation",
+                            help="groupby aggregation function to retrieve")
         parser.add_argument("--start",
+                            type=utils.parse_date,
                             help="beginning of the period")
         parser.add_argument("--stop",
+                            type=utils.parse_date,
                             help="end of the period")
         parser.add_argument("--granularity",
-                            help="granularity to retrieve (in seconds)")
+                            help="granularity to retrieve")
         parser.add_argument("--needed-overlap", type=float,
                             help=("percent of datapoints in each "
                                   "metrics required"))
@@ -231,6 +324,15 @@ class CliMeasuresAggregation(lister.Lister):
         parser.add_argument("--groupby",
                             action='append',
                             help="Attribute to use to group resources"),
+        parser.add_argument("--refresh", action="store_true",
+                            help="force aggregation of all known measures")
+        parser.add_argument("--resample",
+                            help=("granularity to resample time-series to "
+                                  "(in seconds)"))
+        parser.add_argument("--fill",
+                            help=("Value to use when backfilling timestamps "
+                                  "with missing values in a subset of series. "
+                                  "Value should be a float or 'null'."))
         return parser
 
     def take_action(self, parsed_args):
@@ -243,12 +345,15 @@ class CliMeasuresAggregation(lister.Lister):
             metrics=metrics,
             query=parsed_args.query,
             aggregation=parsed_args.aggregation,
+            reaggregation=parsed_args.reaggregation,
             start=parsed_args.start,
             stop=parsed_args.stop,
             granularity=parsed_args.granularity,
             needed_overlap=parsed_args.needed_overlap,
             resource_type=parsed_args.resource_type,
             groupby=parsed_args.groupby,
+            refresh=parsed_args.refresh,
+            resample=parsed_args.resample, fill=parsed_args.fill
         )
         if parsed_args.groupby:
             ms = []
@@ -257,7 +362,7 @@ class CliMeasuresAggregation(lister.Lister):
                                        for k in sorted(g['group']))
                 for m in g['measures']:
                     i = [group_name]
-                    i.extend(m)
+                    i.extend(self.format_measures_with_tz(parsed_args, [m])[0])
                     ms.append(i)
             return ('group',) + self.COLS, ms
-        return self.COLS, measures
+        return self.COLS, self.format_measures_with_tz(parsed_args, measures)
